"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mobiusRange = void 0;
const number_theory_1 = require("number-theory");
function mobiusRange(n1, n2, primalityTest = number_theory_1.miller) {
	const n = n2 - n1;
	const A = new Array(n);
	primalityTest = primalityTest || number_theory_1.miller;
	for (let i = 0; i < n; i++) {
		A[i] = 1;
	}
	let sqrt = Math.ceil(Math.sqrt(n2));
	for (let p = 2; p < sqrt; p++) {
		if (p < n1) {
			if (!primalityTest(p)) {
				continue;
			}
		} else if (A[p - n1] !== 1) {
			continue;
		}
		let i1 = Math.ceil(n1 / p);
		let i2 = Math.ceil(n2 / p);
		for (let i = i1; i < i2; i++) {
			let k = i * p;
			if (i % p === 0) {
				A[k - n1] = 0;
			} else {
				A[k - n1] *= -p;
			}
		}
	}
	for (let i = 0; i < n; i++) {
		if (A[i] === n1 + i) {
			A[i] = 1;
		} else if (A[i] === -n1 - i) {
			A[i] = -1;
		} else if (A[i] < 0) {
			A[i] = 1;
		} else if (A[i] > 0) {
			A[i] = -1;
		} else {
			A[i] = 0;
		}
	}
	if (n1 === 0 && n2 > 0) {
		A[0] = 0;
	}
	return A;
}
exports.mobiusRange = mobiusRange;
//# sourceMappingURL=mobiusRange.js.map
